@@ -102,6 +102,20 @@ class WindowsISCSIapi(base.ConnectionManager):
         if isinstance(auth, iscsiapi_auth.NoAuth):
             return ISCSI_NO_AUTH_TYPE
 
+    def _remove_persistent_target(self, target):
+        self._create_initiator_obj_if_needed()
+        for endpoint in target.get_endpoints():
+            args = ['iscsicli', 'RemovePersistentTarget', str(self._initiator.get_initiator_name()), str(target.get_iqn()),
+                                    '*', str(endpoint.get_ip_address()), str(endpoint.get_port())]
+            logger.info("running {}".format(args))
+            execute(args)
+
+    def _reomve_target_portal(self, target):
+        for endpoint in target.get_endpoints():
+            args = ['iscsicli', 'RemoveTargetPortal', str(endpoint.get_ip_address()), str(endpoint.get_port())]
+            logger.info("running {}".format(args))
+            execute(args)
+
     def discover(self, ip_address, port=3260):
         '''perform an iscsi discovery to an ip address
         '''
@@ -133,15 +147,11 @@ class WindowsISCSIapi(base.ConnectionManager):
                 return session
 
     def _iscsicli_login(self, login_command, target, endpoint, auth=None, num_of_connections=1):
-        def _remove_outbound_secret():
-            cmd = ['iscsicli', 'CHAPSecret', '*']
-            execute(cmd)
 
         auth_type = self._return_auth_type(auth)
         if auth_type == ISCSI_CHAP_AUTH_TYPE:
             username = auth.get_inbound_username()
             password = auth.get_inbound_secret()
-            _remove_outbound_secret()
         elif auth_type == ISCSI_MUTUAL_CHAP_AUTH_TYPE:
             username = auth.get_inbound_username()
             password = auth.get_inbound_secret()
@@ -150,7 +160,7 @@ class WindowsISCSIapi(base.ConnectionManager):
         elif auth_type == ISCSI_NO_AUTH_TYPE:
             username = '*'
             password = '*'
-            _remove_outbound_secret()
+
         # Due to a bug only in 2008 multiple sessions isn't handled ok unless initiator name is monitored
         # Therefore we don't use Qlogin, Details:
         # https://social.technet.microsoft.com/Forums/office/en-US/4b2420d6-0f28-4d12-928d-3920896f582d/iscsi-initiator-target-not-reconnecting-on-reboot?forum=winserverfiles
@@ -272,7 +282,6 @@ class WindowsISCSIapi(base.ConnectionManager):
     def get_discovered_targets(self):
         '''return a list of discovered target objects
         '''
-        import re
         logger.info("get_discovered_targets")
         discovered_targets = []
         client = WmiClient('root\\wmi')
@@ -285,13 +294,17 @@ class WindowsISCSIapi(base.ConnectionManager):
                     if endpoint not in endpoints:
                         endpoints.append(endpoint)
 
-            regex = re.compile(r'(?P<ip>\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\ (?P<port>\d+)')
-            discovery_endpoint = regex.search(query.Properties_.Item('DiscoveryMechanism').Value).groupdict()
-            if discovery_endpoint == []:
-                raise RuntimeError("couldn't find an expected wmi object")
-            discovery_endpoint['port'] = int(str(discovery_endpoint['port']), base=10)
-            target = base.Target(endpoints,
-                                 base.Endpoint(discovery_endpoint['ip'], str(discovery_endpoint['port'])), iqn)
+            # u'SendTargets:*test 0003260 ROOT\\ISCSIPRT\\0000_0 '
+            discovery_mechanism = query.Properties_.Item('DiscoveryMechanism').Value
+            if not discovery_mechanism:
+                logger.debug("invalid discovery mechanism: {!r}".format(discovery_mechanism))
+                continue
+            if discovery_mechanism.strip().count(' ') != 2:
+                logger.debug("invalid discovery mechanism: {!r}".format(discovery_mechanism))
+                continue
+            discovery_endpoint_ip, discovery_endpoint_port, _ = discovery_mechanism.strip().split()
+            discovery_endpoint = base.Endpoint(discovery_endpoint_ip.split(':')[-1], int(str(discovery_endpoint_port), base=10))
+            target = base.Target(endpoints, discovery_endpoint, iqn)
             if target not in discovered_targets:
                 discovered_targets.append(target)
         return discovered_targets
@@ -303,11 +316,8 @@ class WindowsISCSIapi(base.ConnectionManager):
         # TODO: when InfiniBox will support MCS need to modify this code
         from infi.dtypes.hctl import HCT
         logger.info("get_sessions(target={!r}".format(target))
-        def _get_sessions_of_target(target, retries=3):
-            from .iscsi_exceptions import WMIConnectionInformationMissing
-            if not retries:
-                raise WMIConnectionInformationMissing()
 
+        def _get_sessions_of_target(target):
             client = WmiClient('root\\wmi')
             wql = "SELECT * from MSiSCSIInitiator_SessionClass where TargetName='%s'" % str(target.get_iqn())
             query = client.execute_query(wql)
@@ -315,10 +325,11 @@ class WindowsISCSIapi(base.ConnectionManager):
             for session in query:
                 hct = None
                 uid = session.Properties_.Item('SessionId').Value
+                if not uid:
+                    continue
                 connections = session.Properties_.Item('ConnectionInformation').Value
                 if not connections:
-                    sleep(1)
-                    return _get_sessions_of_target(target, retries-1)
+                    continue
 
                 conn_0 = connections[0]
                 source_ip = conn_0.Properties_.Item('InitiatorAddress').Value
@@ -354,18 +365,13 @@ class WindowsISCSIapi(base.ConnectionManager):
         '''
         if target:
             self.logout_all(target)
-            for endpoint in self._get_discovery_endpoints():
-                if endpoint in target.get_endpoints():
-                    args = ['iscsicli', 'RemoveTargetPortal', str(endpoint.get_ip_address()), str(endpoint.get_port())]
-                    logger.info("running {}".format(args))
-                    execute(args)
+            self._reomve_target_portal(target)
+            self._remove_persistent_target(target)
         else:
             for target in self.get_discovered_targets():
                 self.logout_all(target)
-            for endpoint in self._get_discovery_endpoints():
-                args = ['iscsicli', 'RemoveTargetPortal', str(endpoint.get_ip_address()), str(endpoint.get_port())]
-                logger.info("running {}".format(args))
-                execute(args)
+                self._reomve_target_portal(target)
+                self._remove_persistent_target(target)
         self._refresh_wmi_db()
 
 class MicrosoftSoftwareInitiator(base.SoftwareInitiator):
